using System.Security.Permissions;
using System.Net;
using System.Net.Cache;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Request.Courses;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Request;
using System.Linq;
using WingEducationService.Common;

namespace WingEducationService.Service
{
    /// <summary>
    /// 课程考试服务
    /// </summary>
    public partial class EducationService : JsonRpcService, IEducationService
    {

        /// <summary>
        /// 发布考试或者练习试卷接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> PublishCourseExaminationPaperAsync(PublishCourseExaminationPaperRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;//试卷编码
            var courseCode = request.CourseCode;
            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            var existExam = existCourse.BindExams.FirstOrDefault(v => v.Code == code);
            if (existExam == null)
            {
                ThrowRpcException(CustomerRpcCode.ExaminationPaperNoFound, "Examination no found");
            }
            existExam.IsRelease = true;
            return await _courseDBService.UpdateCourseBindExamsAsync(new UpdateCourseBindExamsRequest()
            {
                CourseCode = courseCode,
                BindExams = existCourse.BindExams
            });
        }

        /// <summary>
        /// 查询课程考试列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<BaseCourseExaminationDTO>> FindCourseExaminationPapersAsync(FindCourseExaminationPapersRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //返回已发布试卷
            var bindExams = existCourse.BindExams.Where(c => c.IsRelease).ToList();
            return bindExams.MappingTo<List<BaseCourseExaminationDTO>>();
        }

        /// <summary>
        /// 查询课程考试详情接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseExaminationDTO> FindCourseExaminationPaperByCodeAsync(FindCourseExaminationPaperByCodeRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            var code = request.Code;
            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //检查
            var existExam = existCourse.BindExams.FirstOrDefault(c => c.Code == code);
            if (existExam == null)
            {
                ThrowRpcException(CustomerRpcCode.ExaminationPaperNoFound, "Examination no found");
            }
            return existExam;
        }

        /// <summary>
        /// 提交课程考试接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SubmitCourseExaminationPaperAsync(SubmitCourseExaminationPaperRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            var code = request.Code; //考试编码
            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //检查
            var existExam = existCourse.BindExams.FirstOrDefault(c => c.Code == code);
            if (existExam == null)
            {
                ThrowRpcException(CustomerRpcCode.ExaminationPaperNoFound, "Examination no found");
            }
            var result = await _educationDBService.FindStudentExaminationPagesDBAsync(new FindStudentExaminationPagesDBRequest()
            {
                UserCode = userCode,
                PageIndex = 1,
                PageSize = 10,
                ExaminationCode = code,
                RelevanceCode = courseCode,
                RelevanceType = RelevanceTypeEnum.Course
            });
            var studentExaminationCode = "";
            //学生作答答案
            var answers = request.Answers.MappingTo<List<ExaminationAnswerDTO>>();
            foreach (var item in answers)
            {
                item.QuestionType = existExam.QuestionList.FirstOrDefault(x => x.Code == item.QuestionCode).QuestionType;
            }
            if (result == null || result.TotalCount <= 0)
            {
                studentExaminationCode = await _educationDBService.CreateStudentExaminationDBAsync(new AddStudentExaminationDBRequest()
                {
                    StudentExamination = new StudentExaminationDTO()
                    {
                        RelevanceCode = courseCode,
                        RelevanceType = RelevanceTypeEnum.Course,
                        StudentAnswers = answers,
                        SubmitCount = 1,
                        StudentCode =userCode,
                        ExaminationCode=code
                    }
                });
            }
            else
            {
                var studentExamination = result.PageData.FirstOrDefault();
                if (studentExamination != null)
                {
                    //检查提交次数
                    if (studentExamination.SubmitCount >= existExam.SubmitLimitCount && existExam.SubmitLimitCount != 0)
                    {
                        ThrowRpcException(CustomerRpcCode.SubmitCountError, "Submit Time Error");
                    }
                    studentExamination.StudentAnswers = answers;
                    studentExamination.SubmitCount += 1;
                    await _educationDBService.UpdateStudentExaminationDBAsync(new UpdateStudentExaminationDBRequest()
                    {
                        StudentExamination = studentExamination
                    });
                    studentExaminationCode = studentExamination.Code;
                }
            }
            //自动判题
            AutoJudging(studentExaminationCode);
            return true;
        }

        /// <summary>
        /// 上传远程考核图像
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UploadRemoteExaminationDataAsync(UploadRemoteExamRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            string deviceUniqueCode = request.DeviceUniqueCode;
            //设备物理唯一码必填
            if (string.IsNullOrWhiteSpace(deviceUniqueCode))
            {
                ThrowRpcException(CustomerRpcCode.UniqueCodeIsEmpty, "UniqueCodeIsEmpty");
            }
            var deviceInfo = await _deviceDBService.FindDeviceInfoBySerialNumberAsync(deviceUniqueCode);
            if (deviceInfo == null || string.IsNullOrWhiteSpace(deviceInfo.DeviceCode))
            {
                ThrowRpcException(CustomerRpcCode.UniqueCodeIsError, "UniqueCodeIsError");
            }
            if (string.IsNullOrWhiteSpace(request.FileToken))
            {
                ThrowCustomerException(CustomerRpcCode.FileTokenIsEmpty, "FileToken is empty");
            }
            if (string.IsNullOrWhiteSpace(request.PreviewFileToken))
            {
                ThrowCustomerException(CustomerRpcCode.PreviewFileTokenIsEmpty, "PreviewFileToken is empty");
            }
            if (string.IsNullOrWhiteSpace(request.CoverImageToken))
            {
                ThrowCustomerException(CustomerRpcCode.CoverImageTokenIsEmpty, "CoverImageToken is empty");
            }
            var dataDTO = new RemoteExaminationDTO();
            dataDTO.DeviceCode = deviceInfo.DeviceCode;
            dataDTO.Application = request.Application;
            var originImageUrl = request.FileToken;
            dataDTO.OriginImageUrl = originImageUrl;//源站地址
            dataDTO.ImageUrl = request.FileToken.ToUrlToken();//CDN 地址
            dataDTO.PreviewUrl = request.PreviewFileToken.ToUrlToken();
            dataDTO.CoverImageUrl = request.CoverImageToken.ToUrlToken();
            dataDTO.ImageSize = request.FileSize;
            dataDTO.FileDataType = request.FileDataType;
            dataDTO.MeasuredResult = request.MeasuredResult;
            dataDTO.CommentResult = request.CommentResult;
            dataDTO.UserCode = userCode;
            var result = await _educationDBService.CreateRemoteExaminationDTODBAsync(dataDTO);
            return true;
        }

        /// <summary>
        /// 获取个人远程考核图像列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<RemoteExaminationPageDTO>> GetRemoteExaminationList(TokenRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dbRequest = new FindRemoteExaminationListDBRequest { UserCode = userCode };
            var result = await _educationDBService.FindRemoteExaminationListDBAsync(dbRequest);
            return result;
        }

        /// <summary>
        /// 自动判题
        /// </summary>
        private async void AutoJudging(string code)
        {
            var findStudentExaminationByCodeRequest = new FindStudentExaminationByCodeDBRequest { Code = code };
            var studentExamination = await _educationDBService.FindStudentExaminationByCodeDBAsync(findStudentExaminationByCodeRequest);
            if (studentExamination.RelevanceType == RelevanceTypeEnum.Course)
            {
                var courseRequest = new FindCourseByCodeDBRequest { Code = studentExamination.RelevanceCode };
                var course = await _courseDBService.FindCourseByCodeAsync(courseRequest);
                //试题快照
                var examPaper = course.BindExams.FirstOrDefault(x => x.Code == studentExamination.ExaminationCode);
                if(examPaper==null)
                {
                    return;
                }
                foreach (var studentQuestion in studentExamination.StudentAnswers)
                {
                    var question = examPaper.QuestionList.FirstOrDefault(x => x.Code == studentQuestion.QuestionCode);
                    if (question == null)
                    {
                        continue;
                    }
                    //判断对错
                    if (question.QuestionType == QuestionTypeEnum.Judge)
                    {
                        if (question.TrueOrFalse.ToString().ToLower() == studentQuestion.Answer.ToLower())
                        {
                            studentQuestion.IsCorrect = true;
                            studentQuestion.Score = question.Score;
                        }
                        else
                        {
                            studentQuestion.IsCorrect = false;
                            studentQuestion.Score = 0;
                        }
                    }
                    else if (question.QuestionType == QuestionTypeEnum.SingleChoice || question.QuestionType == QuestionTypeEnum.MultipleChoice)
                    {
                        var correctOptionCodes = question.QuestionOptionList.Where(x => x.TrueOrFalse == true).Select(x => x.Code);
                        if (correctOptionCodes.SequenceEqual(studentQuestion.OptionCodeList))
                        {
                            studentQuestion.IsCorrect = true;
                            studentQuestion.Score = question.Score;
                        }
                        else
                        {
                            studentQuestion.IsCorrect = false;
                            studentQuestion.Score = 0;
                        }
                    }
                }
                //计算总分
                studentExamination.TotalScore = studentExamination.StudentAnswers.Sum(x => x.Score);
                //更新
                await _educationDBService.UpdateStudentExaminationDBAsync(new UpdateStudentExaminationDBRequest { StudentExamination = studentExamination });
            }
        }

        /// <summary>
        ///老师查询分页查询学生答题列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<BaseStudentExaminationDTO>> FindCourseStudentExaminationPagesAsync(FindCourseStudentExaminationPagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            var keyword = request.Keyword;

            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            if (existCourse == null)//课程未找到
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            var result = await _educationDBService.FindStudentExaminationPagesDBAsync(new FindStudentExaminationPagesDBRequest()
            {
                Keyword = keyword,
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                RelevanceCode = courseCode,
                RelevanceType = RelevanceTypeEnum.Course
            });
            var examinationPages = new PageResult<BaseStudentExaminationDTO>();
            if (result == null)
            {
                return examinationPages;
            }
            examinationPages.TotalCount = result.TotalCount;
            examinationPages.PageSize = result.PageSize;
            examinationPages.PageIndex = result.PageIndex;
            if (result.PageData != null && result.PageData.Count > 0)
            {
                examinationPages.PageData = result.PageData.MappingTo<List<BaseStudentExaminationDTO>>();
            }
            return examinationPages;

        }

        /// <summary>
        ///老师查询学生课程答题详情 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<StudentExaminationDetailDTO> FindStudentExaminationByCodeAsync(FindStudentExaminationByCodeRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.ExaminationCodeIsEmpty, "Code  is empty");
            }
            var result = await _educationDBService.FindStudentExaminationByCodeDBAsync(new FindStudentExaminationByCodeDBRequest()
            {
                Code = code
            });
            if (result == null)
            {
                ThrowRpcException(CustomerRpcCode.StudentExaminationNoFound, "Student examination data no found");
            }
            var studentExaminationDetailDTO = result.MappingTo<StudentExaminationDetailDTO>();
            if (result.RelevanceType == RelevanceTypeEnum.Course)//课程
            {
                var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = result.RelevanceCode });
                //课程未找到
                if (existCourse == null)
                {
                    ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
                }
                var existExam = existCourse.BindExams.FirstOrDefault(c => c.Code == result.ExaminationCode); //考试数据
                if (existExam == null)
                {
                    ThrowRpcException(CustomerRpcCode.ExaminationPaperNoFound, "Examination no found");
                }
                studentExaminationDetailDTO.Examination = existExam;
            }
            return studentExaminationDetailDTO;
        }

        /// <summary>
        /// 提交批阅试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SubmitReviewStudentExaminationAsync(SubmitReviewStudentExaminationRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.ExaminationCodeIsEmpty, "Code  is empty");
            }
            var result = await _educationDBService.FindStudentExaminationByCodeDBAsync(new FindStudentExaminationByCodeDBRequest()
            {
                Code = code
            });
            if (result == null)
            {
                ThrowRpcException(CustomerRpcCode.StudentExaminationNoFound, "Student examination data no found");
            }
            var teacherAnswers = request.Answers;//老师阅卷
            foreach (var item in result.StudentAnswers)
            {
                var teacherAnswerItem = teacherAnswers.FirstOrDefault(x => x.QuestionCode == item.QuestionCode);
                item.IsCorrect = teacherAnswerItem.IsCorrect;
                item.Score = teacherAnswerItem.Score;
            }
            //计算总分
            result.TotalScore = result.StudentAnswers.Sum(x => x.Score);
            return await _educationDBService.UpdateStudentExaminationDBAsync(new UpdateStudentExaminationDBRequest()
            {
                StudentExamination = result
            });
        }

        /// <summary>
        ///学生查看自己考试结果列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<BaseStudentExaminationDTO>> FindMyCourseStudentExaminationResultsAsync(FindMyCourseStudentExaminationResultPagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseCode = request.CourseCode;
            var keyword = request.Keyword;

            if (string.IsNullOrEmpty(courseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = courseCode });
            if (existCourse == null)//课程未找到
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            var result = await _educationDBService.FindStudentExaminationPagesDBAsync(new FindStudentExaminationPagesDBRequest()
            {
                Keyword = keyword,
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                RelevanceCode = courseCode,
                RelevanceType = RelevanceTypeEnum.Course,
                UserCode = userCode
            });
            var examinationPages = new PageResult<BaseStudentExaminationDTO>();
            if (result == null)
            {
                return examinationPages;
            }
            examinationPages.TotalCount = result.TotalCount;
            examinationPages.PageSize = result.PageSize;
            examinationPages.PageIndex = result.PageIndex;
            if (result.PageData != null && result.PageData.Count > 0)
            {
                examinationPages.PageData = result.PageData.MappingTo<List<BaseStudentExaminationDTO>>();
            }
            return examinationPages;

        }

        /// <summary>
        ///学生查看自己考试结果
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<StudentExaminationDetailDTO> FindMyCourseStudentExaminationResultAsync(FindMyCourseStudentExaminationResultRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.ExaminationCodeIsEmpty, "Code  is empty");
            }
            var result = await _educationDBService.FindStudentExaminationByCodeDBAsync(new FindStudentExaminationByCodeDBRequest()
            {
                Code = code
            });
            if (result == null)
            {
                ThrowRpcException(CustomerRpcCode.StudentExaminationNoFound, "Student examination data no found");
            }
            var studentExaminationDetailDTO = result.MappingTo<StudentExaminationDetailDTO>();
            if (result.RelevanceType == RelevanceTypeEnum.Course)//课程
            {
                var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = result.RelevanceCode });
                //课程未找到
                if (existCourse == null)
                {
                    ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
                }
                var existExam = existCourse.BindExams.FirstOrDefault(c => c.Code == result.ExaminationCode); //考试数据
                if (existExam == null)
                {
                    ThrowRpcException(CustomerRpcCode.ExaminationPaperNoFound, "Examination no found");
                }
                studentExaminationDetailDTO.Examination = existExam;
            }
            return studentExaminationDetailDTO;
        }
    }
}